@@ -2,7 +2,7 @@ from concurrent.futures import ThreadPoolExecutor
 import atexit
 from operator import length_hint
 from tqdm import tqdm
-from typing import Callable, TypeVar, Iterable
+from typing import Callable, TypeVar, Iterable, Optional
 
 
 SHARED_THREAD_POOL = ThreadPoolExecutor()
@@ -19,8 +19,8 @@ V = TypeVar("V")
 def thread_map(
     fn: Callable[[U], V], 
     it: Iterable[U], 
-    desc: str = None, 
-    total: int = None, 
+    desc: Optional[str] = None, 
+    total: Optional[int] = None, 
     unit: str = "it", 
     verbose: bool = False
 ) -> "list[V]":
